#ifndef PROTOS_H
#define PROTOS_H

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>
#include <sys/time.h>

// needed for the TRACKLE_LOOKUP_COMMAND macro
#include <dlfcn.h>
#include <string.h>

extern char **environ;

/* locally saved function pointers */
extern ssize_t (*__writeptr)(int, const void *, size_t);
extern ssize_t (*__readptr)(int, void *, size_t);
extern size_t  (*__fwriteptr)(const void *, size_t, size_t, FILE *);
extern int     (*__openptr)(const char *, int, ...);
extern int     (*__closeptr)(int);
extern int     (*__fcloseptr)(FILE *);
extern int     (*__fprintfptr)(FILE *, const char *, ...);
extern int     (*__snprintfptr)(char *, size_t, const char *, ...);
extern int     (*__vsnprintfptr)(char *, size_t, const char *, va_list);
#ifdef LINUX
extern int     (*__statptr)(int, const char *, struct stat *);
extern int     (*__lstatptr)(int, const char *, struct stat *);
#else
extern int     (*__lstatptr)(const char *, struct stat *);
extern int     (*__statptr)(const char *, struct stat *);
#endif
extern int     (*__sscanfptr)(const char *, const char *, ...);
extern char*   (*__fgetsptr)(char *, int, FILE *);
extern FILE*   (*__fopenptr)(const char *, const char *);
extern size_t  (*__strlenptr)(const char *);
extern int     (*__readlinkptr)(const char *, char *, size_t );
extern char*   (*__getloginptr)(void);
extern int     (*__gettimeofdayptr)(struct timeval *, struct timezone *);
extern void*   (*__memcpyptr)(void *, const void *, size_t);
extern char*   (*__strncpyptr)(char *, const char *, size_t);
extern void*   (*__mallocptr)(size_t);
extern void*   (*__reallocptr)(void *, size_t);
extern void    (*__freeptr)(void *);
extern char*   (*__get_current_dir_name)(void);
extern int     (*__chmodptr)(const char *path, mode_t mode);
extern int     (*__chownptr)(const char *path, uid_t owner, gid_t group);

// these functions are defined in the header fcntl.h, so
// we have to redeclare them here even though you'd ordinarily
// just include that header
int open(const char* path, int oflag, mode_t mode);
int open64(const char* path, int oflag, mode_t mode);
int creat(const char* path, mode_t mode);


// libtrackle
void  libtrackle_init(void);
void  libtrackle_fini(void);
void* trackle_libopen(const char *path);
void* trackle_getfunc(void *handle, const char *name);
int   trackle_funcptr_setup(void);
int   trackle_stat(const char * , struct stat * );


int trackle_logFileAccess(const char path[]);
int trackle_makeIndexEntry(int index, const char path[], char indexPath[]);
int trackle_copyFile(const char sourcePath[], char destPath[]);
int trackle_statFile(const char sourcePath[], char destPath[], int exists, int readable);
int trackle_logFini(void);
int trackle_getNextIndexLine(FILE *indexFile, char *path, char *ID);
int trackle_alreadyLogged(const char path[], char index[]);
int trackle_getNumFiles(void);
int trackle_incNumFiles(void);
void trackle_fixenv(void);

// these are set in libtrackle_init()
extern char *sessionDir, *fileCounter;


// helpful macros
#define msg(args...) (*__fprintfptr)(stderr, ## args)

#define TRACKLE_LOOKUP_COMMAND(t,n,p,e) if (p==NULL) { p=(t)dlsym(RTLD_NEXT,n); if (p==NULL) { perror("looking up command " n); msg("could not find %s in library: %s\n",n,strerror(errno)); return(e); } }


#endif
