# -*- coding: utf-8 -*-
#
# Copyright (C) 2003-2005 Edgewall Software
# Copyright (C) 2003-2005 Daniel Lundin <daniel@edgewall.com>
# All rights reserved.
#
# This software is licensed as described in the file COPYING, which
# you should have received as part of this distribution. The terms
# are also available at http://trac.edgewall.com/license.html.
#
# This software consists of voluntary contributions made by many
# individuals. For the exact contribution history, see the revision
# history and logs, available at http://projects.edgewall.com/trac/.
#
# Author: Daniel Lundin <daniel@edgewall.com>

from trackle.config import  default_dir
from trackle.db import  Table, Column, Index

# Database version identifier. Used for automatic upgrades.
db_version = 21


##
## Database schema
##

schema = [
    # Common
    Table('system', key='name')[
        Column('name'),
        Column('value')],
    Table('permission', key=('username', 'action'))[
        Column('username'),
        Column('action')],
    Table('auth_cookie', key=('cookie', 'ipnr', 'name'))[
        Column('cookie'),
        Column('name'),
        Column('ipnr'),
        Column('time', type='int')],
    Table('session', key=('sid', 'authenticated', 'var_name'))[
        Column('sid'),
        Column('authenticated', type='int'),
        Column('var_name'),
        Column('var_value')],

    # Attachments
    Table('attachment', key=('type', 'id', 'filename'))[
        Column('type'),
        Column('id'),
        Column('filename'),
        Column('size', type='int'),
        Column('time', type='int'),
        Column('description'),
        Column('author'),
        Column('ipnr')],

    # Wiki system
    Table('wiki', key=('name', 'version'))[
        Column('name'),
        Column('version', type='int'),
        Column('time', type='int'),
        Column('author'),
        Column('ipnr'),
        Column('text'),
        Column('comment'),
        Column('readonly', type='int'),
        Column('public', type='int'),
        Index(['time'])],

    # Version control cache
    Table('revision', key='rev')[
        Column('rev'),
        Column('time', type='int'),
        Column('author'),
        Column('message'),
        Index(['time'])],
    Table('node_change', key=('rev', 'path', 'change_type'))[
        Column('rev'),
        Column('path'),
        Column('node_type', size=1),
        Column('change_type', size=1),
        Column('base_path'),
        Column('base_rev'),
        Index(['rev'])],

    # Ticket system
    Table('ticket', key='id')[
        Column('id', auto_increment=True),
        Column('locked'),
        Column('public', type='int'),
        Column('type'),
        Column('time', type='int'),
        Column('changetime', type='int'),
        Column('component'),
        Column('severity'),
        Column('priority'),
        Column('owner'),
        Column('reporter'),
        Column('cc'),
        Column('version'),
        Column('milestone'),
        Column('status'),
        Column('resolution'),
        Column('summary'),
        Column('description'),
        Column('keywords'),
        Index(['time']),
        Index(['status'])],    
    Table('ticket_change', key=('ticket', 'time', 'field'))[
        Column('ticket', type='int'),
        Column('time', type='int'),
        Column('author'),
        Column('field'),
        Column('oldvalue'),
        Column('newvalue'),
        Index(['ticket', 'time'])],
    Table('ticket_custom', key=('ticket', 'name'))[
        Column('ticket', type='int'),
        Column('name'),
        Column('value')],
    Table('enum', key=('type', 'name'))[
        Column('type'),
        Column('name'),
        Column('value')],
    Table('component', key='name')[
        Column('name'),
        Column('owner'),
        Column('description')],
    Table('milestone', key='name')[
        Column('name'),
        Column('due', type='int'),
        Column('completed', type='int'),
        Column('description')],
    Table('version', key='name')[
        Column('name'),
        Column('time', type='int'),
        Column('description')],

    # Shell Logs System
    Table('shellsession', key=('ticket', 'seq', 'version'))[
        Column('ticket', type='int'),
        Column('seq', type='int'),
        Column('version', type='int'),
        Column('author'),
        Column('ipnr'),
        Column('start', type='int'),
        Column('finish', type='int'),
        Column('text')],
    Table('filediff', key=('ticket', 'seq', 'path'))[
        Column('ticket', type='int'),
        Column('seq', type='int'),
        Column('path'),
        Column('start'),
        Column('finish'),
        Column('sstat'),
        Column('fstat'),
        Column('active', type='int')]

]




##
## Default database values
##

# (table, (column1, column2), ((row1col1, row1col2), (row2col1, row2col2)))
data = (
           ('enum',
             ('type', 'name', 'value'),
               (('status', 'new', 1),
                ('status', 'assigned', 2),
                ('status', 'reopened', 3),
                ('status', 'closed', 4),
                ('status', 'unconfirmed', 5),
                ('resolution', 'fixed', 1),
                ('resolution', 'invalid', 2),
                ('resolution', 'wontfix', 3),
                ('resolution', 'duplicate', 4),
                ('resolution', 'worksforme', 5),
                ('priority', 'critical', 1),
                ('priority', 'major', 2),
                ('priority', 'default', 3),
                ('priority', 'minor', 4),
                ('priority', 'trivial', 5),
                ('severity', 'high', 1),
                ('severity', 'medium', 2),
                ('severity', 'low', 3),
                ('ticket_type', 'defect', 1),
                ('ticket_type', 'enhancement', 2),
                ('ticket_type', 'task', 3))),
           ('permission',
             ('username', 'action'),
               (('anonymous', 'LOG_VIEW'),
                ('anonymous', 'FILE_VIEW'),
                ('anonymous', 'WIKI_VIEW'),
                ('anonymous', 'SEARCH_VIEW'),
                ('anonymous', 'REPORT_VIEW'),
                ('anonymous', 'TICKET_VIEW_BASIC'),
                ('anonymous', 'TICKET_CREATE'),
                ('anonymous', 'TICKET_ADD_CC'),
                ('staff', 'TRAC_ADMIN'),
                )),
           ('system',
             ('name', 'value'),
               (('database_version', str(db_version)),))
           )

default_config = \
 (
  ('trac', 'templates_dir', default_dir('templates')),
  ('trac', 'database', 'postgres://user:password@host/dbname'),
  ('trac', 'default_charset', 'iso-8859-15'),
  ('trac', 'default_handler', 'WikiModule'),
  ('trac', 'check_auth_ip', 'true'),
  ('trac', 'ignore_auth_case', 'false'),
  ('trac', 'metanav', 'login,logout,help,about'),
  ('trac', 'mainnav', 'wiki,timeline,roadmap,shellsession,tickets,newticket,search'),
  ('trac', 'permission_store', 'DefaultPermissionStore'),
  ('logging', 'log_type', 'none'),
  ('logging', 'log_file', 'trackle.log'),
  ('logging', 'log_level', 'DEBUG'),
  ('project', 'name', 'My Project'),
  ('project', 'descr', 'My example project'),
  ('project', 'url', 'http://example.com/'),
  ('project', 'icon', 'common/trackle.ico'),
  ('project', 'footer',
   ' Visit the Trackle open source project at<br />'
   '<a href="http://www.sccs.swarthmore.edu/org/trackle/">http://www.sccs.swarthmore.edu/org/trackle/</a>'),
  ('ticket', 'default_version', ''),
  ('ticket', 'default_type', 'defect'),
  ('ticket', 'default_priority', 'default'),
  ('ticket', 'default_severity', 'medium'),
  ('ticket', 'default_milestone', ''),
  ('ticket', 'default_component', ''),
  ('ticket', 'default_public', '1'),
  ('ticket', 'restrict_owner', 'false'),
  ('ticket', 'default_locked', 'False'),
  ('trackle', 'user_fields', 'severity, type'),
  ('trackle', 'explanation_severity', 'This helps staff gauge the importance '
      'of this issue to you, the user. "High" severity means you can\'t get '
      'important work done, or can\'t log in; "medium" means you\'ve found a '
      'problem with an SCCS service or want to request a new piece of '
      'software be installed; "low" covers annoyances or minor hangups that '
      'you would like to see resolved but can work around easily enough. You '
      'may also leave this field blank.'),
  ('trackle', 'userfile', 'conf/trackle.htpasswd'),
  ('trackle', 'tempdir', '/var/tmp/'),
  ('trackle', 'libdir', ''),
  ('trackle', 'statedir', 'cache/trackle/'),
  ('trackle', 'login_redirect', 'wiki/WikiStart'),
  ('header_logo', 'link', 'http://www.sccs.swarthmore.edu/org/trackle/'),
  ('header_logo', 'src', 'common/trackle_banner.png'),
  ('header_logo', 'alt', 'Trackle'),
  ('header_logo', 'width', '150'),
  ('header_logo', 'height', '36'),
  ('attachment', 'max_size', '262144'),
  ('attachment', 'render_unsafe_content', 'false'),
  ('mimeviewer', 'enscript_path', 'enscript'),
  ('mimeviewer', 'php_path', 'php'),
  ('mimeviewer', 'tab_width', '8'),
  ('mimeviewer', 'max_preview_size', '262144'),
  ('notification', 'smtp_enabled', 'false'),
  ('notification', 'smtp_server', 'localhost'),
  ('notification', 'smtp_port', '25'),
  ('notification', 'smtp_user', ''),
  ('notification', 'smtp_password', ''),
  ('notification', 'smtp_always_cc', ''),
  ('notification', 'always_notify_owner', 'false'),
  ('notification', 'always_notify_reporter', 'false'),
  ('notification', 'smtp_from', 'trac@localhost'),
  ('notification', 'smtp_replyto', 'trac@localhost'),
  ('notification', 'mime_encoding', 'none'),
  ('notification', 'allow_public_cc', 'false'),
  ('notification', 'smtp_staff_addr', ''),
  ('timeline', 'default_daysback', '30'),
  ('timeline', 'changeset_show_files', '0'),
  ('timeline', 'ticket_show_details', 'false'),
  ('wiki', 'ignore_missing_pages', 'false'),
)

default_components = ('trackle.About', 'trackle.attachment',
                      'trackle.db.postgres_backend',
                      'trackle.mimeview.enscript', 'trackle.mimeview.patch',
                      'trackle.mimeview.php', 'trackle.mimeview.rst',
                      'trackle.mimeview.silvercity', 'trackle.mimeview.txtl',
                      'trackle.Search',
                      'trackle.ticket.query',
                      'trackle.ticket.roadmap', 'trackle.ticket.web_ui',
                      'trackle.Timeline',
                      'trackle.shelllogs.web_ui',
                      'trackle.wiki.macros', 'trackle.wiki.web_ui',
                      'trackle.web.auth')

