import re
import types

from trackle import util
from trackle.core import *
from trackle.wiki import IWikiSyntaxProvider, Formatter
from trackle.Search import ISearchSource

class ShellLogSystem(Component):
    implements(IWikiSyntaxProvider)
    
    # IWikiSyntaxProvider methods
    
    def get_wiki_syntax(self):
        return []
    
    def get_link_resolvers(self):
        yield ('shellsession', self._format_link)

    def _format_link(self, formatter, ns, file, label):
        href = self.env.href.shellsession('%s' % file)
        return '<a href="%s">%s</a>' % (util.escape(href), label)


def statToDict(stat):
    """
    translates the contents of a .stat or .stat.end file
    to a python dict. takes lines of the form:

      '<key>: <value ..>'

    and creates dict entries <key> => <value>. assumes
    there are no duplicate keys in the stat
    """

    if type(stat) in types.StringTypes:
        stat = stat.splitlines()
    
    out = {}
    for line in stat:
        line = line.strip()
        try:
            (key, val) = line.split(':')
            val = val.strip()
            out[key] = val
        except ValueError:
            if len(line) and line[-1] == ':':
                out[line[:-1]] = ''

            elif len(line):
                out[line] = ''
    
    return out

def interpretStats(sstat, fstat):
    """
    given stat dicts (as returned by statToDict), interpret
    the action taken on the file between those diffs.

    returns a string, 'modified', 'same', 'deleted', 'added', 'temporary'
    """

    if 'special' not in sstat:
        self.env.log.error("missing special in sstat for %s" % self.path)
        return 'modified'
    if 'special' not in fstat:
        self.env.log.error("missing special in fstat for %s" % self.path)
        return 'modified'


    if 'noexist' in sstat['special'] and 'noexist' in fstat['special']:
        return 'temporary'

    elif 'noexist' in sstat['special']:
        return 'added'

    elif 'noexist' in fstat['special']:
        return 'deleted'

    elif 'mtime' in sstat and 'mtime' in fstat and sstat['mtime'] == fstat['mtime'] and \
         'ctime' in sstat and 'ctime' in fstat and sstat['ctime'] == fstat['ctime']:
        return 'same'

    else:
        return 'modified'
